import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:packages_info_platform_interface/option.dart';
import 'package:packages_info_platform_interface/package.dart';
import 'package:packages_info_platform_interface/packages_info_platform_interface.dart';
import 'package:packages_info_platform_interface/method_channel_package_info.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  group('$PackagesInfoPlatform', () {
    test('$MethodChannelPackagesInfo() is the default instance', () {
      expect(PackagesInfoPlatform.instance,
          isInstanceOf<MethodChannelPackagesInfo>());
    });

    test('Cannot be implemented with `implements`', () {
      expect(
        () {
          PackagesInfoPlatform.instance = ImplementsPackagesInfoPlatform();
        },
        throwsA(isInstanceOf<AssertionError>()),
      );
    });

    test('Can be mocked with `implements`', () {
      final ImplementsPackagesInfoPlatformMock mock =
          ImplementsPackagesInfoPlatformMock();
      PackagesInfoPlatform.instance = mock;
    });

    test('Can be extended', () {
      PackagesInfoPlatform.instance = ExtendsPackagesInfoPlatform();
    });
  });

  group('$MethodChannelPackagesInfo()', () {
    const channel = MethodChannel('plugins.purplegraphite.com/packages_info');
    final log = <MethodCall>[];
    channel.setMockMethodCallHandler((MethodCall methodCall) async {
      log.add(methodCall);
      switch (methodCall.method) {
        case 'getInfoOfAllPackages':
          return [
            <String, dynamic>{
              PackageMapKey.displayName.name: 'package_info_example',
              PackageMapKey.buildNumber.name: '1',
              PackageMapKey.packageIdentifier.name:
                  'io.flutter.plugins.packageinfoexample',
              PackageMapKey.version.name: '1.0',
            },
          ];
        default:
          assert(false);
          return null;
      }
    });

    final packageInfo = MethodChannelPackagesInfo();

    tearDown(() {
      log.clear();
    });

    test('getAll', () async {
      await packageInfo.getInstalledPackages();
      expect(
        log,
        <Matcher>[isMethodCall('getInstalledPackages', arguments: null)],
      );
    });
  });
}

/// Illegal implementation of [PackagesInfoPlatform]
class ImplementsPackagesInfoPlatform implements PackagesInfoPlatform {
  // Intentionally declare self as not a mock to trigger the
  // compliance check.
  @override
  bool get isMock => false;

  @override
  Future<Iterable<Package>> getInstalledPackages([
    Options options = const Options(),
  ]) {
    throw UnimplementedError();
  }

  @override
  Future<Package> getPackageInfo(
    String packageIdentifier, [
    Options options = const Options(),
  ]) {
    throw UnimplementedError();
  }
}

/// Example of a legal mock implementation of [PackagesInfoPlatform]
class ImplementsPackagesInfoPlatformMock extends Mock
    implements PackagesInfoPlatform {
  // Intentionally declare self as a mock to trigger the
  // compliance check.
  @override
  bool get isMock => false;
}

/// Legally extends [PackagesInfoPlatform]
class ExtendsPackagesInfoPlatform extends PackagesInfoPlatform {
  @override
  Future<Iterable<Package>> getInstalledPackages([
    Options options = const Options(),
  ]) {
    throw UnimplementedError();
  }

  @override
  Future<Package> getPackageInfo(
    String packageIdentifier, [
    Options options = const Options(),
  ]) {
    throw UnimplementedError();
  }
}
